# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimList,
    HprimDate,
    HprimForeign,
)


class EpicenterSegmentH(HeaderSegment):
    sender_name = HprimString(5)
    version = HprimString(13, default="V1.0")
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class EpicenterSegmentP(Segment):
    patient_id = HprimString(4)
    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    sex = HprimString(9)
    diagnostic = HprimString(19)
    admission_date = HprimDate(24)
    room = HprimString(26)
    location = HprimString(32)

    orders = HprimForeign("O", "patient")


class EpicenterSegmentQ(Segment):
    accession_number = HprimString((3, 2))
    sequence_number = HprimString((3, 3))
    test_id = HprimString((5, 1))
    test_status = HprimString((5, 2))
    result_qualifier = HprimString((5, 5))
    time_qualifier = HprimDate(6)
    starting_date = HprimDate(7)
    end_date = HprimDate(8)


class EpicenterSegmentO(Segment):
    accession_number = HprimString((3, 1))
    isolate_number = HprimString((3, 2))
    organism = HprimString((3, 3))
    test_id = HprimString((5, 4))
    test_consumable_number = HprimString((5, 5))
    priority = HprimString(6)
    collection_date = HprimDate(8)
    receipt_date = HprimDate(15)
    specimen_type = HprimString(16)
    physician_id = HprimString(17)
    report_type = HprimString(26)

    results = HprimForeign("R", "order")


class EpicenterSegmentR(Segment):
    result_type_code = HprimString((3, 4))
    test_number = HprimString((3, 5))
    antibiotic = HprimString((3, 6))
    test_status = HprimString((4, 1))
    result_date_mic = HprimString((4, 2))
    organism_id = HprimString((4, 3))
    resistance_marker1 = HprimString((4, 4))
    resistance_marker2 = HprimString((4, 5))
    resistance_marker3 = HprimString((4, 6))
    resistance_marker4 = HprimString((4, 7))
    resistance_marker5 = HprimString((4, 8))
    resistance_marker6 = HprimString((4, 9))
    resistance_marker7 = HprimString((4, 10))
    resistance_marker8 = HprimString((4, 11))
    resistance_marker9 = HprimString((4, 12))
    resistance_marker10 = HprimString((4, 13))
    status = HprimString(9)
    test_start_date = HprimDate(12)
    result_status_date = HprimList(13)
    instrument_type = HprimString((14, 1))
    instrument_number = HprimString((14, 2))
    instrument_location = HprimString((14, 5))


class EpicenterSegmentC(Segment):
    comment_text = HprimString(4)
    # S: commentaire specimen
    # I: commentaire isolat
    comment_type = HprimString(5)


class EpicenterSegmentL(Segment):
    code = HprimString(3)


class Driver(AstmMessageV2):
    SPEC = {
        "code": "epicenter",
        "label": "Epicenter PHOENIX BD"
    }

    SEGMENT_CLASS_MAP = {
        "H": EpicenterSegmentH,
        "Q": EpicenterSegmentQ,
        "P": EpicenterSegmentP,
        "O": EpicenterSegmentO,
        "R": EpicenterSegmentR,
        "C": EpicenterSegmentC,
        "L": EpicenterSegmentL,
    }

    DEFAULT_SEPARATORS = "|\\^&"

    OPTIONS = [
        AstmMessageV2.Option(
            "epicenter_liste_commentaires",
            label="Transcodage des commentaires du champ C.4 selon le format <commentaire1_epicenter>=<equivalence1>/<commentaire2_epicenter>=<equivalence2>"
        )
    ]

    def __init__(self, epicenter_liste_commentaires="", **kwargs):
        self.epicenter_liste_commentaires = epicenter_liste_commentaires
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = EpicenterSegmentH(type_segment="H", **kwargs)
        message.end = EpicenterSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        msg = self.create(date=context["now"])
        for patient in worklist.get_astm_worklist(
            unique_together=["action"],
            action={"del": worklist.IGNORE, worklist.ELSE: None},
        ):
            seg_p = msg.patients.add(
                patient_id=patient.patient_id,
                last_name=patient.last_name,
                first_name=patient.first_name,
                birthdate=patient.birthdate,
                sex=patient.sex,
                location=patient.location,
            )
            for sample, action in patient.samples:
                seg_p.room = sample.room
                seg_p.admission_date = sample.admission_date
                seg_p.diagnostic = sample.diagnostic
                for order in sample.orders:
                    seg_o = seg_p.orders.add(
                        accession_number=sample.sample_id,
                        test_id=order.code,
                        collection_date=sample.collection_date,
                        specimen_type=sample.specimen,
                    )
                for physician in sample.physicians:
                    seg_o.physician_id = physician.code
        if msg.patients:
            return msg

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(
                    type="patient",
                    sample_id=seg_q.accession_number,
                )
        else:
            worklist = Worklist(context="result")
            for seg_p in message.patients:
                for seg_o in seg_p.orders:
                    sample = worklist.get_or_create(
                        type="patient",
                        sample_id=seg_o.accession_number,
                    )
                    commentaires = []
                    for seg_c in seg_o.commentaires:
                        # Ignorer les commentaires de type E
                        if seg_c.comment_type == "E":
                            continue
                        # Ne traiter uniquement que les commentaires codés
                        # transcodés dans l'option sur la connexion
                        if self.epicenter_liste_commentaires:
                            for commentaire in self.epicenter_liste_commentaires.split("/"):
                                if "=" in commentaire:
                                    if commentaire.split("=")[0] == seg_c.comment_text:
                                        commentaires.append(commentaire.split("=")[1])
                        else:
                            # Si l'option n'est pas activée, traiter tous les
                            # commentaires codés ou non
                            commentaires.append(seg_c.comment_text)
                    # Résulat d'hémoculture
                    liste_atb = []
                    blood_culture_result = False
                    for seg_r in seg_o.results:
                        if seg_r.result_type_code == "GND":
                            blood_culture_result = True
                            sample.results.add(
                                code="{}{}".format(seg_o.test_id, seg_r.result_type_code),
                                value=seg_r.test_status,
                                comments=commentaires,
                                date=datetime.datetime.strptime(
                                    seg_r.result_status_date[0],
                                    "%Y%m%d%H%M%S"
                                ),
                                instrument=seg_r.instrument_type
                            )
                            if seg_r.test_number:
                                sample.results.add(
                                    code="BOTTLE_ID",
                                    value=seg_r.test_number,
                                    date=datetime.datetime.strptime(
                                        seg_r.result_status_date[0],
                                        "%Y%m%d%H%M%S"
                                    ),
                                    instrument=seg_r.instrument_type
                                )
                            if seg_r.result_status_date[0]:
                                # Heure et date du retrait du flacon
                                date_result_determinated = datetime.datetime.strptime(
                                    seg_r.result_status_date[0],
                                    "%Y%m%d%H%M%S"
                                )
                                sample.results.add(
                                    code="{}DETERMINED_DATETIME".format(seg_o.test_id),
                                    value=date_result_determinated.strftime("%Y%m%d%H%M%S"),
                                )
                                sample.results.add(
                                    code="{}DETERMINED_DATE".format(seg_o.test_id),
                                    value=date_result_determinated.strftime("%Y/%m/%d")
                                )
                                sample.results.add(
                                    code="{}DETERMINED_TIME".format(seg_o.test_id),
                                    value=date_result_determinated.strftime("%H:%M:%S"),
                                )
                                sample.results.add(
                                    code="{}DATE_DETERMINATION".format(seg_o.test_id),
                                    value=date_result_determinated.strftime("%d/%m/%Y"),
                                )
                                sample.results.add(
                                    code="{}HEURE_DETERMINATION".format(seg_o.test_id),
                                    value=date_result_determinated.strftime("%H:%M"),
                                )
                                if "Positive" in seg_r.test_status:
                                    # Date et heure de positivité
                                    sample.results.add(
                                        code="{}POSITIVITY_DATETIME".format(seg_o.test_id),
                                        value=date_result_determinated.strftime("%Y%m%d%H%M%S"),
                                    )
                                    # Date de positivité
                                    sample.results.add(
                                        code="{}POSITIVITY_DATE".format(seg_o.test_id),
                                        value=date_result_determinated.strftime("%Y/%m/%d"),
                                    )
                                    sample.results.add(
                                        code="{}POSITIVITE_DATE".format(seg_o.test_id),
                                        value=date_result_determinated.strftime("%d/%m/%Y"),
                                    )
                                    # Heure de positivité
                                    sample.results.add(
                                        code="{}POSITIVITY_TIME".format(seg_o.test_id),
                                        value=date_result_determinated.strftime("%H:%M:%S"),
                                    )
                                    sample.results.add(
                                        code="{}POSITIVITE_HEURE".format(seg_o.test_id),
                                        value=date_result_determinated.strftime("%H:%M"),
                                    )
                                    # Delai de positivité
                                    if seg_r.result_status_date:
                                        delay_positivity = (date_result_determinated - seg_r.test_start_date).total_seconds() / 3600
                                        sample.results.add(
                                            code="TIME_TO_DETECTION_POSITIVITY_{}".format(seg_o.test_id),
                                            value="{:.1f}".format(delay_positivity),
                                        )
                        elif seg_r.result_type_code == "AST":
                            liste_atb.append({
                                "antibiotic": "A_{}".format(seg_r.antibiotic),
                                "mic": seg_r.result_date_mic,
                                "sensitivity": seg_r.organism_id
                            })
                    if liste_atb:
                        sample.results.add(
                            code=seg_o.specimen_type,
                            value=[{
                                "antibiogram": liste_atb,
                                "value": "G_{}".format(seg_o.organism),
                            }],
                            date=seg_o.receipt_date,
                            nom_antibiogramme=seg_r.resistance_marker3,
                            comments=commentaires,
                        )
                    if not blood_culture_result and seg_o.specimen_type:
                        # version classique avec réponse automate
                        sample.results.add(
                            code="GERME_{}_{}".format(
                                seg_o.isolate_number,
                                seg_o.specimen_type,
                            ),
                            value="G_{}".format(seg_o.organism),
                        )
                        # version avec complexe pour l'identification du germe
                        sample.results.add(
                            code="IDENT_{}".format(
                                seg_o.specimen_type
                            ),
                            value={
                                "seq": "0",
                                "type_resultat": "identification_germes",
                                "germes": [{
                                    "rang": seg_o.isolate_number,
                                    "id": "G_{}".format(seg_o.organism),
                                }]
                            }
                        )
        return worklist
